Package: copula
Version: 1.1-6
VersionNote: Last CRAN: 1.1-5 on 2025-02-09, 1.1-4 on 2024-08-12, 1.1-3
        on 2023-12-07
Date: 2025-03-05
Title: Multivariate Dependence with Copulas
Authors@R: c(person("Marius", "Hofert", role = "aut", email = "marius.hofert@uwaterloo.ca",       comment = c(ORCID = "0000-0001-8009-4665"))
	   , person("Ivan", "Kojadinovic", role = "aut", email = "ivan.kojadinovic@univ-pau.fr",  comment = c(ORCID = "0000-0002-2903-1543"))
	   , person("Martin","Maechler", role=c("aut","cre"), email="maechler@stat.math.ethz.ch", comment = c(ORCID = "0000-0002-8685-9910"))
	   , person("Jun", "Yan", role = "aut", email = "jun.yan@uconn.edu",                      comment = c(ORCID = "0000-0003-4401-7296"))
	   , person(c("Johanna", "G."), "Nešlehová", role = "ctb", comment = c("evTestK()", ORCID = "0000-0001-9634-4796"))
	   , person("Rebecca", "Morger", role = "ctb", comment = "fitCopula.ml(): code for free mixCopula weight parameters")
	   )
Depends: R (>= 3.5.0)
Imports: stats, graphics, methods, stats4, Matrix (>= 1.5-0), lattice,
        colorspace, gsl, ADGofTest, stabledist (>= 0.6-4), mvtnorm,
        pcaPP, pspline, numDeriv
Suggests: MASS, KernSmooth, sfsmisc, scatterplot3d, Rmpfr, bbmle,
        knitr, rmarkdown, animation, abind, crop, gridExtra, HAC,
        lcopula, mev, mvnormtest, parallel, partitions, polynom, qrng,
        randtoolbox, rugarch, Runuran, tseries, VGAM, VineCopula, zoo
SuggestsNote: packages {animation, ..., zoo} (last lines above) are
        only used in vignettes, demos and a few tests.
VignetteBuilder: knitr
Enhances: nor1mix, copulaData
SystemRequirements: pdfcrop (part of TexLive) is required to rebuild
        the vignettes.
Description: Classes (S4) of commonly used elliptical, Archimedean,
 extreme-value and other copula families, as well as their rotations,
 mixtures and asymmetrizations. Nested Archimedean copulas, related
 tools and special functions. Methods for density, distribution, random
 number generation, bivariate dependence measures, Rosenblatt transform,
 Kendall distribution function, perspective and contour plots. Fitting of
 copula models with potentially partly fixed parameters, including
 standard errors. Serial independence tests, copula specification tests
 (independence, exchangeability, radial symmetry, extreme-value
 dependence, goodness-of-fit) and model selection based on
 cross-validation. Empirical copula, smoothed versions, and
 non-parametric estimators of the Pickands dependence function.
License: GPL (>= 3) | file LICENCE
Collate: AllClass.R Classes.R AllGeneric.R Auxiliaries.R aux-acopula.R
        asymCopula.R mixCopula.R rotCopula.R Copula.R special-func.R
        amhCopula.R claytonCopula.R frankCopula.R cop_objects.R
        nacopula.R dC-dc.R amhExpr.R An.R archmCopula.R cCopula.R
        claytonExpr.R ellipCopula.R empCopula.R empPsi.R acR.R
        estimation.R evCopula.R evTests.R exchTests.R fgmCopula.R
        fitCopula.R fitLambda.R fitMvdc.R fixedPar.R frankExpr.R
        galambosCopula.R galambosExpr-math.R galambosExpr.R
        ggraph-tools.R pairsRosenblatt.R prob.R gofTrafos.R
        gofEVTests.R gofCopula.R graphics.R gumbelCopula.R gumbelExpr.R
        huslerReissCopula.R huslerReissExpr.R indepCopula.R fhCopula.R
        lowfhCopula.R upfhCopula.R indepTests.R joeCopula.R K.R
        logseries.R mvdc.R margCopula.R matrix_tools.R normalCopula.R
        obs.R opower.R plackettCopula.R plackettExpr.R moCopula.R
        rstable1.R safeUroot.R schlatherCopula.R stable.R timing.R
        tCopula.R tawnCopula.R tawnExpr.R tevCopula.R
        varianceReduction.R wrapper.R xvCopula.R zzz.R
Encoding: UTF-8
URL: https://copula.r-forge.r-project.org/,
        https://r-forge.r-project.org/projects/copula/,
        https://CRAN.r-project.org/package=copula
BugReports: https://r-forge.r-project.org/tracker/?func=add&group_id=2140&atid=5417
NeedsCompilation: yes
Packaged: 2025-03-18 09:43:59 UTC; maechler
Author: Marius Hofert [aut] (<https://orcid.org/0000-0001-8009-4665>),
  Ivan Kojadinovic [aut] (<https://orcid.org/0000-0002-2903-1543>),
  Martin Maechler [aut, cre] (<https://orcid.org/0000-0002-8685-9910>),
  Jun Yan [aut] (<https://orcid.org/0000-0003-4401-7296>),
  Johanna G. Nešlehová [ctb] (evTestK(),
    <https://orcid.org/0000-0001-9634-4796>),
  Rebecca Morger [ctb] (fitCopula.ml(): code for free mixCopula weight
    parameters)
Maintainer: Martin Maechler <maechler@stat.math.ethz.ch>
Repository: CRAN
Date/Publication: 2025-03-18 13:30:06 UTC
Built: R 4.3.3; aarch64-apple-darwin20; 2025-03-18 13:55:19 UTC; unix
Archs: copula.so.dSYM
