archmCPdf.m: Mathematica code that generates expressions of Archimedean copula
             cdf, pdf, and genfun derivatives; the maximum dimension for Frank
             is determined to be 4, otherwise, the system run out of memory.
             In particular, derPdfWrtPar is the toughest one to get.

evcopula.nb: Mathematica notebook that generates expressions of extreme value
             copula cdf, pdf, and deriv1cdf

exprPrep.R: R code that reads the expression files and generate R objects
            into files ../../R/*Expr.R .

	    The input of algr2dump is a copula which has cdf and pdf expressions
            in exprdist. It generates these files: claytonExpr.R,
	    gumbelExpr.R, frankExpr.R, amhExpr.R, plackettExpr.R,
	    galambosExpr.R, huslerReissExpr.R.


getDerExpr.m: Abandoned Mathematica code that generates derivative expressions.
              Reason of abandoning: after reading pdf expressions back in,
              they are not exactly the same as before. Why?
