---
title: Archimedean Liouville Copulas
author: Marius Hofert
date: '`r Sys.Date()`'
output:
  html_vignette:
    css: style.css
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Archimedean Liouville Copulas}
---
```{r prelim, echo=FALSE}
## lower resolution - less size  (default dpi = 72):
knitr::opts_chunk$set(dpi = 48)
```{r pkg+sourc, message=FALSE}
require(copula)
source(system.file("Rsource", "AC-Liouville.R", package="copula"))
set.seed(271)
```

## Archimedean-Simplex copulas

```{r rACsimp}
n <- 1000
theta <- 0.59
d <- 3
U <- rACsimplex(n, d=d, theta=theta, Rdist="Gamma")
cor(U, method="kendall")
```

```{r pairs-rACsimp, fig.align="center", fig.width=6, fig.height=6}
par(pty="s")
pairs(U, gap=0, pch=".") # or cex=0.5
```


## Liouville copulas

See McNeil, Neslehova (2010, Figure 3)

```{r Liouville}
n <- 2000
theta <- 0.6
alpha <- c(1, 5, 20)
U <- rLiouville(n, alpha=alpha, theta=theta, Rdist="Gamma")
cor(U, method="kendall")
```

```{r pairs-Liouville, fig.align="center", fig.width=6, fig.height=6}
par(pty="s")
pairs(U, gap=0, pch=".") # or cex=0.5
```


## Archimedean-Liouville copulas

See McNeil, Neslehova (2010, Figure 4)

```{r ACLiou}
n <- 1000
theta <- 0.59
alpha <- c(1, 3, 4)
U <- rACLiouville(n, alpha=alpha, theta=theta, family="Clayton")
cor(U, method="kendall")
```

```{r pairs-ACLiou, fig.align="center", fig.width=6, fig.height=6}
par(pty="s")
pairs(U, gap=0, pch=".") # or cex=0.5
```

